"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const micro_1 = require("micro");
const aws_serverless_micro_1 = __importDefault(require("aws-serverless-micro"));
const pino_http_1 = __importDefault(require("pino-http"));
exports.default = (route) => {
    const fn = (Req, Res) => micro_1.run(Req, Res, (req, res) => __awaiter(this, void 0, void 0, function* () {
        const middleware = route.middleware || [];
        if (fn.log !== false) {
            const pinoOptions = process.env.NODE_ENV === 'production' ? {} : {
                prettyPrint: {
                    levelFirst: true,
                },
            };
            const logger = pino_http_1.default(pinoOptions);
            middleware.unshift(logger);
        }
        for (const mw of middleware) { // eslint-disable-line
            yield mw(req, res); // eslint-disable-line
        }
        return route.handler(req, res);
    }));
    fn.path = route.path;
    fn.log = true;
    fn.module = __dirname;
    fn.handler = fn;
    const isAWS = !!((process.env.LAMBDA_TASK_ROOT && process.env.AWS_EXECUTION_ENV) || false);
    if (isAWS) {
        return {
            handler: aws_serverless_micro_1.default(fn),
        };
    }
    const isGCP = !!(process.env.X_GOOGLE_FUNCTION_NAME || false);
    if (isGCP) {
        return {
            handler: fn,
        };
    }
    return fn;
};
